import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

public class ShapedFrame extends JFrame {
  private static final long serialVersionUID = 1L;

  public static boolean isPerpixelTranslucencySupported(final GraphicsConfiguration configuration) {
    return configuration.getDevice().isWindowTranslucencySupported(GraphicsDevice.WindowTranslucency.PERPIXEL_TRANSPARENT);
  }

  public static boolean isPerpixelTransparencySupported(final GraphicsConfiguration configuration) {
    return configuration.getDevice().isWindowTranslucencySupported(GraphicsDevice.WindowTranslucency.PERPIXEL_TRANSPARENT);
  }

  public static boolean isTranslucencyCapable(final GraphicsConfiguration configuration) {
    return configuration.isTranslucencyCapable();
  }

  public static boolean isTranslucencySupported(final GraphicsConfiguration configuration) {
    return configuration.getDevice().isWindowTranslucencySupported(GraphicsDevice.WindowTranslucency.TRANSLUCENT);
  }

  public static void main(final String[] args) throws Exception {
    final GraphicsConfiguration defaultConfiguration = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
    if ((ShapedFrame.isTranslucencyCapable(defaultConfiguration))) {
      if ((ShapedFrame.isPerpixelTranslucencySupported(defaultConfiguration))) {
        if (ShapedFrame.start(defaultConfiguration)) {
          return;
        }
      }
    }
    for (final GraphicsDevice device : GraphicsEnvironment.getLocalGraphicsEnvironment().getScreenDevices()) {
      if ((ShapedFrame.isPerpixelTranslucencySupported(defaultConfiguration))) {
        for (final GraphicsConfiguration configuration : device.getConfigurations()) {
          if (ShapedFrame.start(configuration)) {
            return;
          }
        }
      }
    }
  }

  protected static boolean start(final GraphicsConfiguration configuration) throws Exception {
    if (!ShapedFrame.isTranslucencySupported(configuration)) {
      return false;
    }
    if (!ShapedFrame.isPerpixelTranslucencySupported(configuration)) {
      return false;
    }
    if (!ShapedFrame.isPerpixelTransparencySupported(configuration)) {
      return false;
    }
    if (!ShapedFrame.isTranslucencyCapable(configuration)) {
      return false;
    }
    new ShapedFrame(configuration).setVisible(true);
    return true;
  }

  ShapedFrameUtils.Result result;
  private JPanel pnlBack;

  public ShapedFrame() throws Exception {
    super();
    this.init();
  }

  public ShapedFrame(final GraphicsConfiguration configuration) throws Exception {
    super(configuration);
    this.init();
  }

  protected void init() throws Exception {
    this.setUndecorated(true);
    this.setLocationRelativeTo(null);
    this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
    final MoveMouseListener mml = new MoveMouseListener(this);
    this.addMouseListener(mml);
    this.addMouseMotionListener(mml);
    this.result = ShapedFrameUtils.restore("girl");
    this.setMinimumSize(this.result.size);
    this.setMaximumSize(this.result.size);
    this.setResizable(false);
    this.setSize(this.result.size);
    this.setLayout(new BorderLayout());
    this.pnlBack = new JPanel() {
      private static final long serialVersionUID = 1L;

      @Override
      public void paintComponent(final Graphics g) {
        if ((ShapedFrame.this.result != null) && (ShapedFrame.this.result.image != null)) {
          g.drawImage(ShapedFrame.this.result.image, 0, 0, null);
        } else {
          super.paint(g);
        }
      }
    };
    this.pnlBack.setLayout(new GridBagLayout());
    this.add(this.pnlBack, BorderLayout.CENTER);
    final ShapedButton btnClose = new ShapedButton();
    btnClose.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(final ActionEvent e) {
        ShapedFrame.this.dispose();
      }
    });
    final GridBagConstraints constraints = new GridBagConstraints();
    constraints.gridx = 1;
    constraints.gridy = 1;
    this.pnlBack.add(btnClose, constraints);
    this.setWindowOpaque(false);
    this.setOpacity(.99f);
    this.setShape(this.result.shape);
  }

  public void setWindowOpaque(final boolean b) {
    this.setBackground(new Color(b ? this.getBackground().getRGB() & 0x00FFFFFF : this.getBackground().getRGB() | 0xFF000000));
  }
}
